@@ -8,14 +8,11 @@
 #ifndef __WVCRASH_H
 #define __WVCRASH_H
 
-#include <wvcallback.h>
-
-typedef WvCallback<void, int> WvCrashCallback;
+#include <sys/types.h>
 
 void wvcrash_setup(const char *_argv0, const char *_desc = 0);
 void wvcrash(int sig);
 void wvcrash_add_signal(int sig);
-WvCrashCallback wvcrash_set_callback(WvCrashCallback callback);
 
 // Leave a last will and testament in the WvCrash, if your program dies.
 void wvcrash_leave_will(const char *will);
@@ -24,4 +21,10 @@ const char *wvcrash_read_will();
 // Read the assertion back.
 const char *wvcrash_read_assert();
 
+const int wvcrash_ring_buffer_order = 12;
+const int wvcrash_ring_buffer_size = 1 << wvcrash_ring_buffer_order;
+void wvcrash_ring_buffer_put(const char *str);
+void wvcrash_ring_buffer_put(const char *str, size_t len);
+const char *wvcrash_ring_buffer_get();
+
 #endif // __WVCRASH_H
