@@ -1,6 +1,6 @@
 /* -*- Mode: C++ -*-
  * Worldvisions Weaver Software:
- *   Copyright (C) 1997-2002 Net Integration Technologies, Inc.
+ *   Copyright (C) 1997-2005 Net Integration Technologies, Inc.
  *
  * Various little file functions...
  *
@@ -12,7 +12,6 @@
 #include "wvstring.h"
 #include "wvstringlist.h"
 
-
 /**
  * Create a directory and any subdirectories required along the way.
  * (Equivalent to mkdir -p).
@@ -39,6 +38,12 @@ void rm_rf(WvStringParm _dir);
 bool fcopy(WvStringParm src, WvStringParm dst);
 bool fcopy(WvStringParm srcdir, WvStringParm dstdir, WvStringParm relname);
 
+/**
+ * Create the file if it doesn't exist, or update the file's modification and
+ * access times if it does.
+ * (Equivalent to touch).
+ */
+bool ftouch(WvStringParm file, time_t mtime = 0);
 
 /**
  * Check whether two files have the same date/time stamp. This can be used as a
@@ -64,7 +69,6 @@ bool wvfnmatch(WvStringList &patterns, WvStringParm name, int flags = 0);
  */
 FILE *wvtmpfile();
 
-
 /* Returns a unique filename suitable for a temporary file. Obviously there is
  * the caveat that someone else may claim this file name before you open it:
  * do not use this routine where that race may be a real concern (this would
@@ -72,7 +76,6 @@ FILE *wvtmpfile();
  */
 WvString wvtmpfilename(WvStringParm prefix);
 
-
 /**
  * Basically our own implementation of the NetBSD lchmod() call.
  */
@@ -80,7 +83,6 @@ WvString wvtmpfilename(WvStringParm prefix);
 int wvchmod(const char *path, mode_t mode);
 #endif
 
-
 /**
  * A simple helper function to get the current umask.
  */
