@@ -56,6 +56,9 @@ public:
      * return localaddr() instead.
      */
     virtual const WvUnixAddr *src() const;
+    
+public:
+    const char *wstype() const { return "WvUnixConn"; }
 };
 
 /** Server end of a Unix Sockets stream */
@@ -106,6 +109,9 @@ protected:
     void *auto_userdata;
 
     static void accept_callback(WvStream &s, void *userdata);
+
+public:
+    const char *wstype() const { return "WvUnixListener"; }
 };
 
 
