@@ -106,6 +106,30 @@ bool fcopy(WvStringParm srcdir, WvStringParm dstdir, WvStringParm relname)
 }
 
 
+bool ftouch(WvStringParm file, time_t mtime)
+{
+    if (!WvFile(file, O_WRONLY|O_CREAT).isok())
+        return false;
+
+    struct utimbuf *buf = NULL;
+    if (mtime != 0)
+    {
+        buf = (struct utimbuf *)malloc(sizeof(struct utimbuf));
+        buf->actime = time(NULL);
+        buf->modtime = mtime;
+    }
+
+    if (utime(file, buf) == 0)
+    {
+        free(buf);
+        return true;
+    }
+
+    free(buf);
+    return false;
+}
+
+
 bool samedate(WvStringParm file1, WvStringParm file2)
 {
     struct stat buf;
