@@ -7,6 +7,12 @@
 #include "wvfile.h"
 #include "wvmoniker.h"
 
+WvFile::WvFile()
+{
+    readable = writable = false;
+    skip_select = false;
+}
+
 #ifndef _WIN32 // meaningless to do this on win32
 /*
  * The Win32 runtime library doesn't provide fcntl so we can't
@@ -37,17 +43,17 @@ WvFile::WvFile(WvStringParm filename, int mode, int create_mode)
 }
 
 
-static IWvStream *increator(WvStringParm s, IObject *, void *)
+static IWvStream *increator(WvStringParm s)
 {
     return new WvFile(s, O_RDONLY, 0666);
 }
 
-static IWvStream *outcreator(WvStringParm s, IObject *, void *)
+static IWvStream *outcreator(WvStringParm s)
 {
     return new WvFile(s, O_WRONLY|O_CREAT|O_TRUNC, 0666);
 }
 
-static IWvStream *creator(WvStringParm s, IObject *, void *)
+static IWvStream *creator(WvStringParm s)
 {
     return new WvFile(s, O_RDWR|O_CREAT, 0666);
 }
@@ -90,6 +96,36 @@ bool WvFile::open(WvStringParm filename, int mode, int create_mode)
     return true;
 }
 
+#ifndef _WIN32  // since win32 doesn't support fcntl
+
+bool WvFile::open(int _rwfd)
+{
+    noerr();
+    if (_rwfd < 0)
+        return false;
+
+    noerr();
+    close();
+    
+    int mode = fcntl(_rwfd, F_GETFL);
+    int xmode = (mode & (O_RDONLY | O_WRONLY | O_RDWR));
+    readable = (xmode == O_RDONLY) || (xmode == O_RDWR);
+    writable = (xmode == O_WRONLY) || (xmode == O_RDWR);
+
+    skip_select = false;
+    
+    if (!readable)
+	undo_force_select(true, false, false);
+
+    setfd(_rwfd);
+    fcntl(_rwfd, F_SETFL, mode | O_NONBLOCK);
+    fcntl(_rwfd, F_SETFD, 1);
+
+    closed = stop_read = stop_write = false;
+    return true;
+}
+
+#endif 
 
 // files not open for read are never readable; files not open for write
 // are never writable.
