@@ -27,7 +27,7 @@ static WvMoniker<IWvStream> reg("clone", creator);
 
 
 WvStreamClone::WvStreamClone(IWvStream *_cloned) 
-    : cloned(0), disassociate_on_close(false)
+    : cloned(0), disassociate_on_close(false), my_type("WvStreamClone:(none)")
 {
     setclone(_cloned);
     // the sub-stream will force its own values, if it really wants.
@@ -176,8 +176,11 @@ void WvStreamClone::setclone(IWvStream *newclone)
     closed = stop_read = stop_write = false;
     if (cloned)
 	cloned->setclosecallback(IWvStreamCallback(this, &WvStreamClone::close_callback));
-    if (newclone)
-        closed = stop_read = stop_write = false;
+    
+    if (newclone != NULL)
+        my_type = WvString("WvStreamClone:%s", newclone->wstype());
+    else
+        my_type = "WvStreamClone:(none)";
 }
 
 
