@@ -4,13 +4,24 @@
  *
  * Various little time functions...
  */
+#include <limits.h>
+
 #include "wvtimeutils.h"
 
 time_t msecdiff(const WvTime &a, const WvTime &b)
 {
-    time_t secdiff = a.tv_sec - b.tv_sec;
-    time_t usecdiff = a.tv_usec - b.tv_usec;
-    return secdiff * 1000 + usecdiff / 1000;
+    long long secdiff = a.tv_sec - b.tv_sec;
+    long long usecdiff = a.tv_usec - b.tv_usec;
+    long long msecs = secdiff * 1000 + usecdiff / 1000;
+
+    time_t rval;
+    if (msecs > INT_MAX)
+	rval = INT_MAX;
+    else if (msecs < INT_MIN)
+	rval = INT_MIN;
+    else
+	rval = msecs;
+    return rval;
 }
 
 
@@ -50,3 +61,14 @@ WvTime tvdiff(const WvTime &a, const WvTime &b)
     return c;
 }
 
+static WvTime wvstime_cur = wvtime();
+
+const WvTime &wvstime()
+{
+    return wvstime_cur;
+}
+
+void wvstime_sync()
+{
+    wvstime_cur = wvtime();
+}
